@@ -1,25 +1,27 @@
-NUM_VERSION = (2012, 3)
+NUM_VERSION = (2013, 1)
 VERSION = ".".join(str(nv) for nv in NUM_VERSION)
+__version__ = VERSION
 
 from pudb.py3compat import raw_input
 
 
 CURRENT_DEBUGGER = []
-def _get_debugger():
+def _get_debugger(**kwargs):
     if not CURRENT_DEBUGGER:
         from pudb.debugger import Debugger
-        dbg = Debugger()
+        dbg = Debugger(**kwargs)
+
         CURRENT_DEBUGGER.append(dbg)
         return dbg
     else:
         return CURRENT_DEBUGGER[0]
 
-
-
+import signal
+DEFAULT_SIGNAL = signal.SIGINT
+del signal
 
 def runscript(mainpyfile, args=None, pre_run="", steal_output=False):
-    from pudb.debugger import Debugger
-    dbg = Debugger(steal_output=steal_output)
+    dbg = _get_debugger(steal_output=steal_output)
 
     # Note on saving/restoring sys.argv: it's a good idea when sys.argv was
     # modified by the script being debugged. It's a bad idea when it was
@@ -37,10 +39,6 @@ def runscript(mainpyfile, args=None, pre_run="", steal_output=False):
     prev_sys_path = sys.path[:]
     sys.path[0] = dirname(mainpyfile)
 
-    from pudb.settings import load_breakpoints
-    for bpoint_descr in load_breakpoints(dbg):
-        dbg.set_break(*bpoint_descr)
-
     while True:
         if pre_run:
             from subprocess import call
@@ -119,14 +117,34 @@ def set_trace():
     import sys
     dbg = _get_debugger()
 
-    from pudb.settings import load_breakpoints
-    for bpoint_descr in load_breakpoints(dbg):
-        dbg.set_break(*bpoint_descr)
-
+    set_interrupt_handler()
     dbg.set_trace(sys._getframe().f_back)
 
 
+def _interrupt_handler(signum, frame):
+    from pudb import _get_debugger
+    _get_debugger().set_trace(frame)
 
+def set_interrupt_handler(interrupt_signal=DEFAULT_SIGNAL):
+    """
+    Set up an interrupt handler, to activate PuDB when Python receives the
+    signal `interrupt_signal`.  By default it is SIGINT (i.e., Ctrl-c).
+
+    To use a different signal, pass it as the argument to this function, like
+    `set_interrupt_handler(signal.SIGALRM)`.  You can then break your code
+    with `kill -ALRM pid`, where `pid` is the process ID of the Python
+    process.  Note that PuDB will still use SIGINT once it is running to allow
+    breaking running code.  If that is an issue, you can change the default
+    signal by hooking `pudb.DEFAULT_SIGNAL`, like
+
+    >>> import pudb
+    >>> import signal
+    >>> pudb.DEFAULT_SIGNAL = signal.SIGALRM
+
+    Note, this may not work if you use threads or subprocesses.
+    """
+    import signal
+    signal.signal(interrupt_signal, _interrupt_handler)
 
 def post_mortem(exc_info=None):
     if exc_info is None:
