@@ -16,6 +16,18 @@ if PY3:
 else:
     _next = "next"
 
+try:
+    from functools import partial
+except ImportError:
+    def partial(func, *args, **keywords):
+        def newfunc(*fargs, **fkeywords):
+            newkeywords = keywords.copy()
+            newkeywords.update(fkeywords)
+            return func(*(args + fargs), **newkeywords)
+        newfunc.func = func
+        newfunc.args = args
+        newfunc.keywords = keywords
+        return newfunc
 
 HELP_TEXT = """\
 Welcome to PuDB, the Python Urwid debugger.
@@ -44,7 +56,7 @@ Keys:
     m - open module
 
     j/k - up/down
-    ctrl-u/d - page up/down
+    Ctrl-u/d - page up/down
     h/l - scroll left/right
     g/G - start/end
     L - show (file/line) location / go to line
@@ -58,6 +70,8 @@ Keys:
     f1/?/H - show this help screen
     q - quit
 
+    Ctrl-c - when in continue mode, break back to PuDB
+
 Side-bar related:
 
     +/- - grow/shrink sidebar
@@ -89,7 +103,7 @@ License:
 
 PuDB is licensed to you under the MIT/X Consortium license:
 
-Copyright (c) 2009,10,11 Andreas Kloeckner and contributors
+Copyright (c) 2009-13 Andreas Kloeckner and contributors
 
 Permission is hereby granted, free of charge, to any person
 obtaining a copy of this software and associated documentation
@@ -113,8 +127,6 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 """
 
-
-
 # {{{ debugger interface
 
 class Debugger(bdb.Bdb):
@@ -135,6 +147,10 @@ class Debugger(bdb.Bdb):
             self.stolen_output = sys.stderr = sys.stdout = StringIO()
             sys.stdin = StringIO("") # avoid spurious hangs
 
+        from pudb.settings import load_breakpoints
+        for bpoint_descr in load_breakpoints():
+            self.set_break(*bpoint_descr)
+
     def save_breakpoints(self):
         from pudb.settings import save_breakpoints
         save_breakpoints([
@@ -296,6 +312,11 @@ class Debugger(bdb.Bdb):
             statement = 'exec(compile(open("%s").read(), "%s", "exec"))' % (filename, filename)
         else:
             statement = 'execfile( "%s")' % filename
+
+        # Set up an interrupt handler
+        from pudb import set_interrupt_handler
+        set_interrupt_handler()
+
         self.run(statement, globals=globals_, locals=locals_)
 
 # }}}
@@ -398,8 +419,6 @@ class DebuggerUI(FrameVarInfoKeeper):
 
         # }}}
 
-        from functools import partial
-
         def change_rhs_box(name, index, direction, w, size, key):
             from pudb.settings import save_config
 
@@ -918,11 +937,10 @@ class DebuggerUI(FrameVarInfoKeeper):
                         try:
                             __import__(str(new_mod_name))
                         except:
-                            from traceback import format_exception
-                            import sys
+                            from pudb.lowlevel import format_exception
 
                             self.message("Could not import module '%s':\n\n%s" % (
-                                new_mod_name, "".join(format_exception(*sys.exc_info()))),
+                                new_mod_name, "".join(format_exception(sys.exc_info()))),
                                 title="Import Error")
                         else:
                             show_mod(sys.modules[str(new_mod_name)])
@@ -939,11 +957,14 @@ class DebuggerUI(FrameVarInfoKeeper):
                         mod = sys.modules[mod_name]
                         reload(mod)
                         self.message("'%s' was successfully reloaded." % mod_name)
-                elif result == "import":
-                    mod = import_new_module(filt_edit.get_edit_text())
-                    if mod is not None:
-                        show_mod(mod)
-                    break
+
+                        from linecache import clearcache
+                        clearcache()
+
+                        self.set_current_file(self.shown_file, force_update=True)
+
+                        _, pos = self.stack_list._w.get_focus()
+                        self.debugger.set_frame_index(self.translate_ui_stack_index(pos))
 
         self.source_sigwrap.listen("n", next)
         self.source_sigwrap.listen("s", step)
@@ -987,13 +1008,20 @@ class DebuggerUI(FrameVarInfoKeeper):
             raw_input("Hit Enter to return:")
             self.screen.start()
 
+        def reload_breakpoints(w, size, key):
+            self.debugger.clear_all_breaks()
+            from pudb.settings import load_breakpoints
+            for bpoint_descr in load_breakpoints(dbg):
+                dbg.set_break(*bpoint_descr)
+            self.update_breakpoints()
+
         def show_traceback(w, size, key):
             if self.current_exc_tuple is not None:
-                from traceback import format_exception
+                from pudb.lowlevel import format_exception
 
                 result = self.dialog(
                         urwid.ListBox([urwid.Text(
-                            "".join(format_exception(*self.current_exc_tuple)))]),
+                            "".join(format_exception(self.current_exc_tuple)))]),
                         [
                             ("Close", "close"),
                             ("Location", "location")
@@ -1099,6 +1127,7 @@ class DebuggerUI(FrameVarInfoKeeper):
 
 
         self.top.listen("o", show_output)
+        self.top.listen("ctrl r", reload_breakpoints)
         self.top.listen("!", run_shell)
         self.top.listen("e", show_traceback)
 
@@ -1236,6 +1265,58 @@ class DebuggerUI(FrameVarInfoKeeper):
         screen.register_palette(
                 get_palette(may_use_fancy_formats, CONFIG["theme"]))
 
+    def show_exception_dialog(self, exc_tuple):
+        from pudb.lowlevel import format_exception
+
+        tb_txt = "".join(format_exception(exc_tuple))
+        while True:
+            res = self.dialog(
+                    urwid.ListBox([urwid.Text(
+                        "The program has terminated abnormally because of an exception.\n\n"
+                        "A full traceback is below. You may recall this traceback at any "
+                        "time using the 'e' key. "
+                        "The debugger has entered post-mortem mode and will prevent further "
+                        "state changes.\n\n"
+                        + tb_txt)]),
+                    title="Program Terminated for Uncaught Exception",
+                    buttons_and_results=[
+                        ("OK", True),
+                        ("Save traceback", "save"),
+                        ])
+
+            if res in [True, False]:
+                break
+
+            if res == "save":
+                try:
+                    n = 0
+                    from os.path import exists
+                    while True:
+                        if n:
+                            fn = "traceback-%d.txt" % n
+                        else:
+                            fn = "traceback.txt"
+
+                        if not exists(fn):
+                            outf = open(fn, "w")
+                            try:
+                                outf.write(tb_txt)
+                            finally:
+                                outf.close()
+
+                            self.message("Traceback saved as %s." % fn, title="Success")
+
+                            break
+
+                        n += 1
+
+                except Exception:
+                    io_tb_txt = "".join(format_exception(sys.exc_info()))
+                    self.message(
+                            "An error occurred while trying to write the traceback:\n\n"
+                            + io_tb_txt,
+                            title="I/O error")
+
     # }}}
 
     # {{{ UI enter/exit
@@ -1260,6 +1341,7 @@ class DebuggerUI(FrameVarInfoKeeper):
     # }}}
 
     # {{{ interaction
+
     def event_loop(self, toplevel=None):
         prev_quit_loop = self.quit_event_loop
 
@@ -1271,7 +1353,7 @@ class DebuggerUI(FrameVarInfoKeeper):
                 self.message("Package 'pygments' not found. "
                         "Syntax highlighting disabled.")
 
-        WELCOME_LEVEL = "e007"
+        WELCOME_LEVEL = "e008"
         if CONFIG["seen_welcome"] < WELCOME_LEVEL:
             CONFIG["seen_welcome"] = WELCOME_LEVEL
             from pudb import VERSION
@@ -1285,6 +1367,10 @@ class DebuggerUI(FrameVarInfoKeeper):
                     "look familiar.\n\n"
                     "If you're new here, welcome! The help screen (invoked by hitting "
                     "'?' after this message) should get you on your way.\n"
+                    "\nChanges in version 2013.1:\n\n"
+                    "- Ctrl-C will now break to the debugger in a way that does\n"
+                    "  not terminate the program\n"
+                    "- Lots of bugs fixed\n"
                     "\nChanges in version 2012.3:\n\n"
                     "- Python 3 support (contributed by Brad Froehle)\n"
                     "- Better search box behavior (suggested by Ram Rachum)\n"
@@ -1360,17 +1446,8 @@ class DebuggerUI(FrameVarInfoKeeper):
             % VERSION)]
 
         if self.debugger.post_mortem:
-            from traceback import format_exception
-
             if show_exc_dialog:
-                self.message(
-                        "The program has terminated abnormally because of an exception.\n\n"
-                        "A full traceback is below. You may recall this traceback at any "
-                        "time using the 'e' key. "
-                        "The debugger has entered post-mortem mode and will prevent further "
-                        "state changes.\n\n"
-                        + "".join(format_exception(*exc_tuple)),
-                        title="Program Terminated for Uncaught Exception")
+                self.show_exception_dialog(exc_tuple)
 
             caption.extend([
                 (None, " "),
@@ -1385,11 +1462,11 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.caption.set_text(caption)
         self.event_loop()
 
-    def set_current_file(self, fname):
+    def set_current_file(self, fname, force_update=False):
         from pudb.source_view import SourceLine, format_source
         fname = self.debugger.canonic(fname)
 
-        if self.shown_file != fname:
+        if self.shown_file != fname or force_update:
             if fname == "<string>":
                 self.source[:] = [SourceLine(self, fname)]
             else:
@@ -1411,10 +1488,9 @@ class DebuggerUI(FrameVarInfoKeeper):
                     self.source[:] = format_source(self,
                             decoded_lines, set(breakpoints))
                 except:
-                    from traceback import format_exception
-
+                    from pudb.lowlevel import format_exception
                     self.message("Could not load source file '%s':\n\n%s" % (
-                        fname, "".join(format_exception(*sys.exc_info()))),
+                        fname, "".join(format_exception(sys.exc_info()))),
                         title="Source Code Load Error")
                     self.source[:] = [SourceLine(self,
                         "Error while loading '%s'." % fname)]
@@ -1423,7 +1499,7 @@ class DebuggerUI(FrameVarInfoKeeper):
             self.current_line = None
 
     def show_line(self, line, fname=None):
-        chaged_file = False
+        changed_file = False
         if fname is not None:
             changed_file =  self.shown_file != fname
             self.set_current_file(fname)
@@ -1505,11 +1581,11 @@ class DebuggerUI(FrameVarInfoKeeper):
         self.stack_walker[:] = frame_uis
 
     def show_exception(self, exc_type, exc_value, traceback):
-        from traceback import format_exception
+        from pudb.lowlevel import format_exception
 
         self.message(
                 "".join(format_exception(
-                    exc_type, exc_value, traceback)),
+                    (exc_type, exc_value, traceback))),
                 title="Exception Occurred")
 
     # }}}
