@@ -124,7 +124,6 @@ class VariableWidget(urwid.FlowWidget):
             apfx = self.attr_prefix+" "
 
         var_label = self.var_label or ''
-        value_str = self.value_str or ''
 
         if self.wrap:
             text = self._get_text(size)
@@ -334,22 +333,12 @@ class ValueWalker:
 
             # class types -------------------------------------------------
             key_its = []
-            try:
-                key_its.append(value.__slots__)
-            except:
-                pass
 
             try:
-                key_its.append(value.__dict__.iterkeys())
+                key_its.append(dir(value))
             except:
                 pass
 
-            if not key_its:
-                try:
-                    key_its.append(dir(value))
-                except:
-                    pass
-
             keys = [key for key_it in key_its for key in key_it]
             keys.sort()
 
